@@ -1,5 +1,5 @@
 /// <reference types="tree-sitter-cli/dsl" />
-const {
+import {
   key_val_arg,
   maybe_bang,
   keyword,
@@ -8,7 +8,13 @@ const {
   commaSep1,
   sep1,
   command,
-} = require("./rules/utils");
+} from "./rules/utils.js";
+import { keywords } from "./rules/keywords.js";
+import autocmd from "./rules/autocmd.js";
+import highlight from "./rules/highlight.js";
+import syntax from "./rules/syntax.js";
+import cmd from "./rules/command.js";
+import edit from "./rules/edit.js";
 
 const MAP_OPTIONS = any_order(
   "<buffer>",
@@ -94,7 +100,7 @@ const PREC = {
   CALL: 8, //expr[n] expr[n:m] expr.name expr(...)
 };
 
-module.exports = grammar({
+export default grammar({
   name: "vim",
 
   word: ($) => $.keyword,
@@ -104,6 +110,7 @@ module.exports = grammar({
     [$.binary_operation, $.field_expression],
     [$._ident, $.lambda_expression],
     [$._ident, $._immediate_lambda_expression],
+    [$._complete_range_marker, $._range_marker]
   ],
 
   externals: ($) =>
@@ -123,7 +130,7 @@ module.exports = grammar({
       $.comment,
       $.line_continuation_comment,
       $._bang_filter,
-    ].concat(require("./keywords").keywords($)),
+    ].concat(keywords($)),
 
   extras: ($) => [$._line_continuation, $.line_continuation_comment, /[\t ]/],
 
@@ -166,6 +173,7 @@ module.exports = grammar({
           $.throw_statement,
           $.autocmd_statement,
           $.silent_statement,
+          $.tab_statement,
           $.vertical_statement,
           $.belowright_statement,
           $.aboveleft_statement,
@@ -204,12 +212,15 @@ module.exports = grammar({
           $.visual_statement,
           $.view_statement,
           $.eval_statement,
-          $.user_command
+          $.substitute_statement,
+          $.user_command,
+          $.shebang,
         )
       ),
 
     unknown_builtin_statement: ($) =>
       seq(
+        optional(field("range", alias($._complete_range, $.range))),
         maybe_bang($, $.unknown_command_name),
         alias(repeat($.command_argument), $.arguments)
       ),
@@ -307,7 +318,10 @@ module.exports = grammar({
     colorscheme_statement: ($) =>
       command($, "colorscheme", optional(alias($.filename, $.name))),
 
-    lua_statement: ($) => command($, "lua", choice($.chunk, $.script)),
+    lua_statement: ($) => choice(
+      command($, "lua", choice($.chunk, $.script)),
+      seq("=", choice($.chunk, $.script))
+    ),
     ruby_statement: ($) => command($, "ruby", choice($.chunk, $.script)),
     python_statement: ($) => command($, "python", choice($.chunk, $.script)),
     perl_statement: ($) => command($, "perl", choice($.chunk, $.script)),
@@ -319,6 +333,7 @@ module.exports = grammar({
     script: ($) =>
       seq(
         "<<",
+        optional(alias("trim", $.parameter)),
         choice(alias($._script_heredoc_marker, $.marker_definition), "\n"),
         alias(repeat($._heredoc_line), $.body),
         alias($._heredoc_end, $.endmarker)
@@ -481,6 +496,7 @@ module.exports = grammar({
           $.register,
           $.option,
           $.index_expression,
+          $.slice_expression,
           $.field_expression,
           $.list_assignment
         ),
@@ -573,6 +589,7 @@ module.exports = grammar({
     execute_statement: ($) => command($, "execute", repeat1($._expression)),
 
     silent_statement: ($) => command_modifier($, "silent", true),
+    tab_statement: ($) => command_modifier($, "tab", false),
     vertical_statement: ($) => command_modifier($, "vertical", false),
     topleft_statement: ($) => command_modifier($, "topleft", false),
     botright_statement: ($) => command_modifier($, "botright", false),
@@ -591,10 +608,13 @@ module.exports = grammar({
 
     user_command: ($) =>
       seq(
+        optional(field("range", alias($._complete_range, $.range))),
         maybe_bang($, $.command_name),
         alias(repeat($.command_argument), $.arguments)
       ),
 
+    shebang: ($) => seq('#!', optional(/.+/)),
+
     command_argument: ($) => choice($.string_literal, /\S+/),
 
     function_definition: ($) =>
@@ -669,6 +689,27 @@ module.exports = grammar({
         $.mark
       ),
 
+    _complete_range: ($) =>
+      choice(alias("%", $.file), $._complete_range_explicit),
+
+    _complete_range_explicit: ($) =>
+      seq(
+        field("start", $._complete_range_marker),
+        optional(seq(choice(",", ";"), field("end", $._complete_range_marker)))
+      ),
+
+    _complete_range_marker: ($) =>
+      choice(
+        $.integer_literal,
+        $.current_line,
+        $.next_line,
+        $.last_line,
+        seq("/", $.pattern, token.immediate("/")),
+        seq("?", $.pattern, token.immediate("?")),
+        $.previous_pattern,
+        $.mark
+      ),
+
     current_line: ($) => ".",
     next_line: ($) => "+",
     last_line: ($) => "$",
@@ -1053,6 +1094,8 @@ module.exports = grammar({
 
     eval_statement: ($) => command($, "eval", $._expression),
 
+    substitute_statement: ($) => range_command($, "substitute", $.pattern),
+
     _method_call_expression: ($) =>
       seq(
         field(
@@ -1227,11 +1270,21 @@ module.exports = grammar({
         )
       ),
 
-    ...require("./rules/autocmd"),
-    ...require("./rules/command"),
-    ...require("./rules/highlight"),
-    ...require("./rules/syntax"),
-    ...require("./rules/edit"),
+    ...Object.fromEntries(
+      Object.entries(autocmd).map(([key, fn]) => [key, $ => fn($)])
+    ),
+    ...Object.fromEntries(
+      Object.entries(cmd).map(([key, fn]) => [key, $ => fn($)])
+    ),
+    ...Object.fromEntries(
+      Object.entries(highlight).map(([key, fn]) => [key, $ => fn($)])
+    ),
+    ...Object.fromEntries(
+      Object.entries(syntax).map(([key, fn]) => [key, $ => fn($)])
+    ),
+    ...Object.fromEntries(
+      Object.entries(edit).map(([key, fn]) => [key, $ => fn($)])
+    ),
   },
 });
 
@@ -1244,7 +1297,7 @@ function command_modifier($, name, bang) {
 }
 
 function _cmd_range($) {
-  return seq(field("range", alias($._range, $.range)), optional(":"));
+  return seq(field("range", alias($._complete_range, $.range)), optional(":"));
 }
 function range_command($, cmd, ...args) {
   return seq(optional(_cmd_range($)), keyword($, cmd), ...args);
